/*
(c) Janusz Ganczarski
http://www.januszg.hg.pl
JanuszG@enter.net.pl
*/

// interpolowany wektor normalny we wsprzdnych obserwatora

varying vec3 N;

// interpolowane wsprzdne biecego wierzchoka

varying vec3 V;

// interpolowany wektor kierunku rda wiata

varying vec3 L;

// interpolowane skadowe wiata rozproszonego,
// otaczajcego i odbitego (bez wykadnika rozbysku)

varying vec4 diffuse,ambient,global_ambient,specular;

// interpolowany wykadnik rozbysku wiata

varying float shininess;

// interpolowana odlego biecego wierzchoka od pooenia rda wiata

varying float distance;

void main ()
{
    // normalizacja interpolowanego wektora normalnego
    N = normalize (N);

    // normalizacja interpolowanego wektora kierunku rda wiata
    L = normalize (L);

    // obliczenie wspczynnika rozproszenia wiata
    float att = 1.0 / (gl_LightSource[0].constantAttenuation +
                gl_LightSource[0].linearAttenuation * distance +
                gl_LightSource[0].quadraticAttenuation * distance * distance);

    // obliczenie cosinusa kta pomidzy wektorem normalnym a wektorem
    // kierunku wiata; poniewa oba wektory s jednostkowe ich iloczyn
    // skalarny jest rwny poszukiwanemu cosinusowi
    float NdotL = max (dot (N,L),0.0);

    // obliczenia zwizane ze wiatem odbitym i rozproszonym 
    // wykonywane s tylko dla owietlonych powierzchni
    if (NdotL > 0.0)
    {
        // obliczenie i normalizacja wektora kierunku kamery;
        // vec3 (0.0) okrela wsprzdne pooenia obserwatora
        vec3 E = normalize (vec3 (0.0) - V);

        // obliczenie wektora odbicia
        vec3 R = normalize (-reflect (L,N));

        // obliczenie cosinusa kta pomidzy wektorem kamery a wektorem odbicia
        float RdotE = max (dot (R,E),0.0);

        // obliczenie wartoci skadowych wiata odbitego
        specular = specular * pow (RdotE,shininess);

        // dodanie wartoci skadowych wiata rozproszonego 
        specular = specular + diffuse * NdotL;
    }
    else
        // warto skadowych wiata odbitego i rozproszonego dla nieowietlonej powierzchni
        specular = vec4 (0.0);

    // kocowe obliczenie skadowych koloru fragmentu
    gl_FragColor = att * (ambient + specular) + global_ambient;
}
